package auth

import (
	"fmt"
	"github.com/dgrijalva/jwt-go"
	"github.com/gin-gonic/gin"
	"net/http"
	"time"
)

const (
	CtxRequestHeaderUserId        = "user_id"
	ctxRequestHeaderAuthorization = "Authorization"
	ctxRequestCookieAuthorization = "ak"
	ctxRequestTokenExpired        = "expired"
)

func Auth(authKey string) gin.HandlerFunc {
	return func(ctx *gin.Context) {
		var (
			err error
			tk  = ctx.Request.Header.Get(ctxRequestHeaderAuthorization)
		)

		if tk == "" {
			tk, err = ctx.Cookie(ctxRequestCookieAuthorization)
			if err != nil {
				ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed"})
				return
			}

			tk = "Bearer " + tk
		}

		if len(tk) < 8 {
			ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed"})
			return
		}

		token, err := jwt.Parse(tk[7:], func(token *jwt.Token) (interface{}, error) {
			if _, ok := token.Method.(*jwt.SigningMethodHMAC); !ok {
				return nil, fmt.Errorf("unexpected signing method: %v", token.Header["alg"])
			}

			return []byte(authKey), nil
		})

		if err != nil || !token.Valid {
			ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed"})
			return
		}

		if mapClaims, ok := token.Claims.(jwt.MapClaims); ok {
			if expired, ok := mapClaims[ctxRequestTokenExpired].(float64); ok {
				switch true {
				case expired > 0:
					if int64(expired) < time.Now().Unix() {
						ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed, token timeout"})
						return
					}

					// todo check expired from server
				case expired == 0:
					// Only cookie is exists, check token expired. app expired by itself call logout when app exit
					if _, err := ctx.Cookie(ctxRequestCookieAuthorization); err != nil {
						ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed, token timeout"})
						return
					}

				default:
					ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed, token timeout"})
					return
				}

				if uid, ok := mapClaims[CtxRequestHeaderUserId].(float64); ok {
					ctx.Set(CtxRequestHeaderUserId, int64(uid))
				} else {
					ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed, mapClaims[CtxRequestHeaderUserId].(float64) error"})
					return
				}
			} else {
				ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed, mapClaims[ctxRequestTokenExpired].(float64) error"})
				return
			}
		} else {
			ctx.AbortWithStatusJSON(http.StatusUnauthorized, gin.H{"msg": "auth failed, token.Claims.(jwt.MapClaims) error"})
			return
		}
	}
}
